#==============================================================================
#
#  $Id: addrbk.py,v 1.2 2000/07/16 17:46:45 mike Exp $
#
"""
   Address book data object classes.
"""
#
#  Copyright (C) 1999 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#  $Log: addrbk.py,v $
#  Revision 1.2  2000/07/16 17:46:45  mike
#  Implemented the creation of address groups.
#
#  Revision 1.1  2000/04/30 21:07:22  mike
#  Added initial address book window.
#
#
#==============================================================================

import shelve

class AddressBookEntry:
   
   """
      Base class of all address book entries.
      
      Public-vars:
         nickname::
            [string] Short name by which the entry is referred.
   """
   
   def __init__(self, nickname):
      "parms: (@AddressBookEntry).Public-vars"
      self.nickname = nickname
   
   def getDisplayName(self):
      """
         Returns the display name of the entry - the name that will be
         printed out in the address list for all references to the entry.
      """
      raise NotImplementedError()

class AddressEntry(AddressBookEntry):

   """
      An entry that corresponds to a particular e-mail address.
      
      Public-vars:
         lastName::
            [string] last name of the owner of the address (or full name of
	    a non-person entity).
         firstName::
            [string] first name of the owner of the address (empty string if
            this is a non-person entity).
         emailAddress::
            [string] a standard SMTP-style e-mail address.
         see-also: (@AddressBookEntry).Public-vars
   """
   
   def __init__(self, nickname, lastName, firstName, emailAddress):
      "parms: (@AddressEntry).Public-vars"
      AddressBookEntry.__init__(self, nickname)
      self.lastName = lastName
      self.firstName = firstName
      self.emailAddress = emailAddress
   
   def getDisplayName(self):
      "Implements @AddressBookEntry.getDisplayName()"
      if self.firstName:
         return '%s %s' % (self.firstName, self.lastname)
      else:
         return self.lastName

   def asAddress(self):
      """
         Returns the entry as a full e-mail address of the form "first-name
         last-name <e-mail-address>"
      """
      if self.firstName:
         return '%s %s <%s>' % \
            (self.firstName, self.lastName, self.emailAddress)
      else:
         return '%s <%s>' % (self.lastName, self.emailAddress)

   def __cmp__(self, other):
      return cmp(self.lastName, other.lastName) or \
         cmp(self.firstName, other.firstName) or \
         cmp(self.nickname, other.nickname)

class GroupEntry(AddressBookEntry):
   
   """
      An entry corresponding to a group of users.
      
      Public-vars:
         fullName::
            [string] Full group name for display purposes
         members::
            [list<string>] list of nicknames of all users
         see-also: (@AddressBookEntry).Public-vars
   """
   
   def __init__(self, nickname, fullName, members):
      "parms: (@GroupEntry).Public-vars"
      AddressBookEntry.__init__(self, nickname)
      self.fullName = fullName
      self.members = members
   
   def getDisplayName(self):
      return self.fullName

class AddressBook:
   """

   """   
   def __init__(self, path):
      """
         parms:
            path::
               [string] full path to the address book database.
      """      
      self.__shelf = shelve.open(path)
      self.__all = {}
      
      # address book should be small enough to fit in memory
      for key in self.__shelf.keys():
         self.__all[key] = self.__shelf[key]
   
   def __getitem__(self, key):
      return self.__all[key]
   
   def __setitem__(self, key, val):
      self.__shelf[key] = self.__all[key] = val
   
   def __delitem__(self, key):
      del self.__shelf[key], self.__all[key]

   def has_key(self, key):
      return self.__all.has_key(key)

   def getAllAddresses(self):
      """
         Returns a list of all @AddressEntry items in the book.
      """
      list = []
      for v in self.__all.values():
         if isinstance(v, AddressEntry):
            list.append(v)
      return list
   
   def getAllGroups(self):
      """
         Returns a list of all @GroupEntry items in the book.
      """
      list = []
      for v in self.__all.values():
         if isinstance(v, GroupEntry):
            list.append(v)
      return list
   

                             


   