#==============================================================================
#
#  $Id: behavior.py,v 1.8 2001/03/31 23:06:38 mike Exp $
#
"""
   Module to house the @Behavior class.
"""
#
#  Copyright (C) 1999 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#  $Log: behavior.py,v $
#  Revision 1.8  2001/03/31 23:06:38  mike
#  Implemented message ids.
#
#  Revision 1.7  2000/06/25 21:36:10  mike
#  Added message forwarding.
#
#  Revision 1.6  2000/05/30 12:42:34  mike
#  Added address mangling, made deletion of user during reply default behavior.
#
#  Revision 1.5  2000/05/11 01:00:32  mike
#  Dates are now generated for replies; fixed timezone code and day of month.
#
#  Revision 1.4  2000/04/30 21:07:22  mike
#  Added initial address book window.
#
#  Revision 1.3  2000/04/27 23:15:27  mike
#  Added auto-file filters
#
#  Revision 1.2  2000/04/20 23:28:18  mike
#  Minor fixes.
#
#  Revision 1.1  2000/04/17 12:36:24  mike
#  New behavior module: base class for custom.
#
#
#==============================================================================

import string, os, time

# imported just so that we can generate a unique message id
import whrandom, socket

from senders import SMTPSender
from message import Message, MessageHeaders

class AddressRule:
   """
      Base class for all rules used in an @AddressMangler.
      
      Address rules can be used to automatically transform e-mail addresses
      in a message.
   """

   def __init__(self):
      pass
   
   def matches(self, name, email):
      """
         Returns true if the rule matches the user name and e-mail address.
      """
      raise NotImplementedError()
   
   def action(self, name, email):
      """
         Returns a tuple for the new name and e-mail address, or just
         *None* if the address should be removed from the list.
      """
      raise NotImplementedError()

class DeleteAddress(AddressRule):
   """
      An address rule that deletes all addresses matching the address
      supplied upon construction.
   """
   
   def __init__(self, addrToDelete):
      """
         parms:
            addrToDelete::
               [string] the e-mail address to be deleted.
      """
      self.addr = string.lower(addrToDelete)
   
   def matches(self, name, email):
      email = string.lower(email)
      return email == self.addr
   
   def action(self, name, email):
      return None   

class AddressMangler:
   
   """
      An address mangler applies a set of @AddressRule instances to an
      e-mail message, allowing you to transform and remove addresses.
      
      Normally used as a filter.
   """

   def __init__(self, rules):
      """
         parms:
            rules::
               [list<@AddressRule>] list of address rules to appply.
      """
      self.rules = rules

   def __call__(self, msg):
      headers = msg.headers
      
      # list of headers that will be deleted as a result of our changes
      toDelete = []
      
      for i in range(len(headers)):
         headerName = headers[i][0]
         if headerName in ('to', 'cc', 'bcc'):
            addrlist = headers.asAddressList(i)
            newAddrList = []
            changed = 0
            for name, email in addrlist.addresslist:
               for rule in self.rules:
                  if rule.matches(name, email):
                     newItem = rule.action(name, email)
                     if newItem:
                        newAddrList.append(newItem)
                     changed = 1
                     break
               else:
                  newAddrList.append((name, email))
            
            if changed:
               # if all addresses were removed, mark this header for deletion.
               if not newAddrList:
                  toDelete.append(i)
               else:
                  # Convert the address list to a string.
                  newAddrList = map(lambda x: '"%s" <%s>' % x, newAddrList)
                  newAddrList = string.join(newAddrList, ', ')
                  headers[i] = headerName, newAddrList

      # delete all headers that were reduced to nothing
      for i in toDelete:
         del headers[i]



class Behavior:
   
   """
      This class holds all of the business logic.  
      
      Users should override from this class in their ".spugmailrc" file,
      replacing any behaviors necessary.      
      
      Protected-vars:
         `_loaders`::
            [list<@spug.mail.loaders.Loader>] List of objects used to
            fetch the mail.
            
         `_replyFilters`::
            [list<callable>] List of callable objects that can be used
            to edit a reply message before it is given to the user in 
            the composer.
            
            By default, this includes a single @AddressMangler instance
            with a @DeleteAddress rule that deletes the address of the
            sender.  If you override this, it is recommended that you
            include this functionality.
            
         `_autoFileFilters`::
            [list<callable>] List of callable objects that are used to
            select a folder for a message when the user hits "auto move".
   """

   def __init__(self):
      self._loaders = []
      manglerRules = [ DeleteAddress(self.getUserAddr()) ]
      self._replyFilters = [ AddressMangler(manglerRules) ]
      self._autoFileFilters = []
      
   def getSender(self):
      """
         Returns the message sender (instance of @spug.mail.senders.Sender
         used to send e-mail messages).
      """
      return SMTPSender()
  
   def getLoaders(self):
      """
         Returns the list of all message loaders (instance of 
         @spug.mail.loaders.Loader used to fetch the mail)
      """
      return self._loaders
   
   def sendMessage(self, msg):
      """
         This is the function that is used to send e-mail messages.  The 
         determination of how to send an e-mail message is done from
         the @getSender() method.  sendMessage() just invokes the
         sendMessage() method on its return value.
      """
      self.getSender().sendMessage(msg)
   
   def getUserAddr(self):
      """
         Returns the user's e-mail address.  By default, this is the user
	 login name (as determined by the "id -un" command) and the hostname
	 (as determined by hostname (as determined by the "hostname"
	 command).
      """
      try:
         loginName = self.__userLoginName
      except AttributeError:
         loginName = self.__userLoginName = \
            string.strip(os.popen('id -un').read())
      
      try:
         hostName = self.__hostName
      except AttributeError:
         hostName = self.__hostName = \
            string.strip(os.popen('hostname').read())
      
      return '%s@%s' % (loginName, hostName)

   def getUserName(self):
      """
         Returns the user's full name.  This is determined by obtaining
         the user id of the process and then parsing the `/etc/passwd` file.
      """
      try:
         return self.__userFullName
      except AttributeError:
      
         # try to find the user from the password file
         uid = os.getuid()
         for line in open('/etc/passwd').readlines():
            fields = string.split(line, ':')
            if fields and int(fields[2]) == uid:
               self.__userFullName = fields[4]
               return self.__userFullName
         else:
            self.__userFullName = 'Unknown User'
            return self.__userFullName

   def getUserAddressString(self):
      """
         Returns the user address string.  By default, this returns
         the full user name (as returned by @getUserName() in quotes 
         followed by the user's e-mail address (as returned by 
         @getUserAddr()) enclosed in angle brackets, for example:
         
         {{
            "User Name" <user@somehost.com>
         }}
      """
      return '"%s" <%s>' % (self.getUserName(), self.getUserAddr())
   
   def getSignature(self):
      """
         Returns the signature as a string.  By default, this reads the
         signature file from ".signature" in the directory identified by
         the "HOME" environment variable.
      """
      home = os.environ['HOME']
      return open('%s/.signature' % home).read()

   def getFolderPath(self):
      """
         Returns the path to the spug mail folder files: by default, this
         is `$HOME/.spugmail/mail`
      """      
      return os.environ['HOME'] + '/.spugmail/mail'

   def getAddressBook(self):
      """
         Returns the path to the spugmail address book database file.  By
         default, this is `$HOME/.spugmail/address`.
      """
      return os.environ['HOME'] + '/.spugmail/address'

   def getReplyFilters(self):
      """
         Returns the list of "reply filters".  These are a list of callable 
         objects that are called by @makeReplyMessage() after it creates 
         a reply message, but before it returns it.  Each object is passed
         in the instance of the new reply message, and may modify it
         according to specific parameters.
         
         By default, this is a list containing a single @AddressMangler
         instance with a @DeleteAddress rule to remove the user's address
         from the list of addresses to reply to.
      """
      return self._replyFilters

   def getAutoFileFilters(self):
      """
         Returns the list of "auto file" filters.  These are a list of 
         callable objects that accept a @spug.mail.message.Messageinstance
         and return either a folder name (if the filter can determine
         that the message belongs in that folder) or *None* (if the filter
         can make no determination).
         
         When the user selects the "auto move" option, the list of filters
         is automatically traversed.  The first filter that returns 
         a folder name is the one that is used.
      """
      return self._autoFileFilters
         
   def makeReplyMessage(self, msg, replyToAll):
      """
         Returns a new reply message for the given message.
         
         parms:
            msg:: 
               [@Message.Message] instance to create a reply message for.
            replyToAll:: 
               set to true if you want the reply message to be to
               all recipients as well as the sender.
      """
      
      # first construct the headers list, which is the "from" address plus
      # (optionally) all of the 'to' and 'cc' addresses
      hdrs = MessageHeaders()
   
      # add the "from" field from the sender
      hdrs['from'] = self.getUserAddressString()
   
      # get the from address and sender name
      try:
         hdrs['to'] = msg.headers['from']
         senderName = msg.headers.asAddressList('from').addresslist[0][0]
      except KeyError, ex:
         # no from field?
         hdrs['to'] = '<no "from" field!>'
         senderName = '<no "from" field!>'

      # optionally get "to" and "cc"
      if replyToAll:
         for hdr, content in msg.headers:
            if hdr in ('to', 'cc'):
               hdrs.addHeader(hdr, content)
   
      # set the date      
      hdrs['date'] = self._makeDate()
   
      # add the subject header
      
      try:
         subject = msg.headers['subject']
      except KeyError, ex:
         subject = "<no subject>"
      
      # make sure it has a "re:" prefix
      if string.lower(subject[:3]) == 're:':
         hdrs['subject'] = subject
      else:
         hdrs['subject'] = 'Re: ' + subject
   
      # now construct the body, which is the original body with "> " in front
      # of every line
      newBody = '\n%s wrote:\n' % senderName
      for line in string.split(msg.getText(), '\n'):
         newBody= '%s> %s\n' % (newBody, line)
   
      # tack on a SIG
      newBody = newBody + '\n' + self.getSignature()

      # create message id
      hdrs['message-id'] = self._makeMessageId()

      # create the new message
      replyMsg = Message(hdrs, newBody, [])
   
      # call the reply filters
      for filter in self.getReplyFilters():
         filter(replyMsg)
      
      return replyMsg

   def _makeMessageId(self):
      """
         Creates a unique message id from the host name, pid, date and
         time and a random number.
      """
      return '<%d.%f.%d@%s>' % \
         (os.getpid(), time.time(), whrandom.randint(0, 2000000000),
          socket.gethostname()
          )
   
   def _makeTimeZone(self, dst):
      """
         Returns a timezone string suitable for use in a mail message.
         
         This will be a 3-4 digit number indicating the number of hours
         and minutes away from GMT we are.
         
         parms:
            dst::
               [int] true or false depending on whether daylight savings
               time is in effect.
      """ 
      
      # figure out which zone to use based on DST      
      if dst:
         zone = time.altzone
      else:
         zone = time.timezone
      
      # convert seconds to military-clock style representation
      zone = -((zone / 3600) * 100 + (zone % 3600) / 60)
      
      # format so that there is always 4 digits
      if zone < 0:
         return '%05d' % zone
      else:
         return '%04d' % zone

   def _makeDate(self):
      """
         Returns a mail message represntation of the current date.
      """
      localTime = time.localtime(time.time())
      zone = self._makeTimeZone(localTime[8])
      return time.strftime('%a, %d %b %Y %H:%M:%S ', localTime) + zone
                                                
   def makeNewMessage(self):
      """
         Returns a brand new message, ready to be composed.
      """
      body = '\n' + self.getSignature()
      hdrs = MessageHeaders()
      # add the "from" field from the sender
      hdrs['from'] = self.getUserAddressString()
            
      # create the date      
      hdrs['date'] = self._makeDate()
                                   
      hdrs['subject'] = ''
      hdrs['message-id'] = self._makeMessageId()
      return Message(hdrs, body, [])
   
   def makeForwardedMessage(self, msg):
      """
         Returns a new message object that is /msg/ with "forward"
         encapsulation.
         
         parms:
            msg:: 
               [@Message.Message] instance to create a reply message for.
      """
      hdrs = MessageHeaders()
      
      # add the user's address
      hdrs['from'] = self.getUserAddressString()
      
      # create the date
      hdrs['date'] = self._makeDate()

      # get the original subject

      try:
         subject = msg.headers['subject']
      except KeyError, ex:
         subject = "<no subject>"
      
      # construct the new subject from it
      hdrs['subject'] = 'Fwd: ' + subject
      
      # construct the body from the plain text of the other message
      body = '\n---------- Forwarded Message ----------\n' + \
         msg.forDisplay(attachmentSummary = 0)
      
      # now create an attachment for the full content of the forwarded
      # message
      fwdHdrs = MessageHeaders()
      fwdHdrs['content-type'] = 'message/rfc822'
      fwdHdrs['content-description'] = 'Forwarded Message'
      fwdHdrs['content-disposition'] = 'attachment'
      fwdHdrs['content-encoding'] = '7bit'
      forwarded = Message(fwdHdrs, str(msg.headers) + '\n' + msg.body,
                          msg.attachments
                          )
      
      hdrs['message-id'] = self._makeMessageId()
      return Message(hdrs, body, [forwarded])

      