#==============================================================================
#
#  $Id: loaders.py,v 1.5 2000/07/10 00:02:27 mike Exp $
#
"""
   This module contains a bunch of "message loaders" - classes that
   go out and retrieve messages from various sources.
"""
#
#  Copyright (C) 1999 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#  $Log: loaders.py,v $
#  Revision 1.5  2000/07/10 00:02:27  mike
#  Made password interface a little more friendly.
#
#  Revision 1.4  2000/04/14 01:19:14  mike
#  Added "quit" to POP3 stuff to clean up deletes.
#
#  Revision 1.3  2000/04/13 23:52:07  mike
#  Added SOCKSPOP3Loader to load messages through the SOCKS firewall.
#
#  Revision 1.2  2000/04/12 00:43:54  mike
#  Added hidden entryfields and initial support for customization.
#
#  Revision 1.1.1.1  2000/04/02 19:09:03  mike
#  Spug Mailer and libraries.
#
#
#==============================================================================

from message import MessageHeaders, Message

import poplib, string

class LoaderInterface:
   
   def storeMessage(self, msg):
      """
         Derived classes must implement this.  Stores a /message/ (an instance
         of @spug.mail.message.Message) in some storage repository.  
         Successful completion of this method is assumed to indicate that it
         is safe to delete the message from the source repository.
      """
      raise NotImplementedError()

   def queryUser(self, prompt, secret):
      """
         Queries the user for something stored. /prompt/ is a string to 
         be distplayed to the user.  /secret/ is an integer indicating
         whether the contents of the entryfield should be hidden.
         
         Must be implemented by subclass.
      """
      raise NotImplementedError()

   def showStatus(self, message):
      """
         Displays a status message to the user.
      """
      raise NotImplementedError()

def parseMessage(messageList):
   """
      Parses a message from /messageList/, a list of strings, and returns
      a @spug.mail.message.Message instance.
   """
   headers = []
   body = []
   
   # build the list of headers from everything up to the empty line, build
   # the body from everything after the empty line
   curList = headers
   for line in messageList:
      if not line and curList is headers:
         curList = body
         continue
         
      curList.append(line)
   
   # parse the headers list
   headersObj = MessageHeaders()
   headersObj.parseFrom(headers)
   
   return Message(headersObj, string.join(body, '\n'))

class POP3Loader:
   
   def __init__(self, host, user, password, deleteMessages = 1):
      self.host = host
      self.user = user
      self.password = password
      self.deleteMessages = deleteMessages

   def _getPOP3(self):
      """   
         Creates the POP3 instance, may be overriden by classes that
         wish to use a modified POP3 object.
      """
      return poplib.POP3(self.host)
   
   def getMessages(self, interface):
      """
         Gets all messages, stores them in /interface/, and returns a list
         of any errors that occurred.
      """

      # first try to connect and send user, immediately fail if there was a 
      # problem      
      try:
         interface.showStatus('Connecting to %s...' % self.host)
         pop = self._getPOP3()
         pop.user(self.user)
      except poplib.error_proto, ex:
         return [ ex[0] ]

      # make three tries at password authentication
      prompt = 'Enter password:'
      for i in range(3):
         try:
            if self.password is None:
               self.password = interface.queryUser(prompt, 1)         
            pop.pass_(self.password)
            break
         except poplib.error_proto, ex:
            prompt = 'Bad login, Enter password:'
            self.password = None
      else:
         return [ ex[0] ]
      
      # keep an error list
      errors = []
      
      # get the message count
      count, size = pop.stat()
      for i in range(1, count + 1):
         interface.showStatus('Getting message %d...' % i)
         
         # try to get the message
         try:
            msgInfo = pop.retr(i)
            msg = parseMessage(msgInfo[1])
            interface.storeMessage(msg)
         except poplib.error_proto, ex:
            errors.append('Error retrieving message %d: %s' % (i, ex[0]))
            continue
         
         # if we are deleting messages, delete this one
         if self.deleteMessages:
            interface.showStatus('Deleting message %d...' % i)
            try:
               pop.dele(i)
            except poplib.error_proto, ex:
               errors.append('Error occurred while deleting message %d: %s' %
                              (i, ex[0])
                             )

      # do the officially sanctioned quit
      try:
         pop.quit()
      except poplib.error_proto, ex:
         errors.append('Error occurred during quit: %s' % ex[0])

      return errors

from spug.net.socks import SOCKSSocket

class SOCKSPOP3(poplib.POP3):
   
   def __init__(self, socksServer, host, port, socksPort, socksUser):
      self.host = host
      self.port = port
      self.sock = SOCKSSocket(socksServer, host, port, socksPort, socksUser)
      self.file = self.sock.reader
      self._debugging = 0
      self.welcome = self._getresp()      

class SOCKSPOP3Loader(POP3Loader):
   """
      Gets POP3 messages through a SOCKS server.
   """
   
   def __init__(self, socksServer, host, user, password, deleteMessages = 1,
                socksPort = 1080, socksUser = ''):
      POP3Loader.__init__(self, host, user, password, deleteMessages)
      self.socksServer = socksServer
      self.socksPort = socksPort
      self.socksUser = socksUser
   
   def _getPOP3(self):
      return SOCKSPOP3(self.socksServer, self.host, 110, self.socksPort, 
                       self.socksUser
                       )

if __name__ == '__main__':
   
   import time
   class Bogus(LoaderInterface):
      
      def showStatus(self, msg):
         print msg
      
      def storeMessage(self, msg):
         info = msg.asMessageInfo(0)
         print 'from:', info.sender
         print 'to:', info.recipients
         print 'date:', time.strftime('%Y-%m-%d %H:%M:%S', info.date)
         print 'subject:', info.subject
         print 'unread:', info.unread
   
   loader = POP3Loader('localhost', 'mike', 'NBOGUE', 0)
   errors = loader.getMessages(Bogus())
   if errors:
      print 'there were errors:'
      for err in errors:
         print '   ', err
      